
/***************************************************************************
 *   This file is part of Aspect, a simple PEC tool.                       *
 *                                                                         *
 *   Copyright (C) 2006-2007 by Wolfgang Hoffmann <woho@woho.de>           *
 *                                                                         *
 *   This program is free software, licensed under the GPL v2.             *
 *   See the file COPYING for more details.                                *
 ***************************************************************************/


#ifndef ASPECT_MCUSERIAL_H
#define ASPECT_MCUSERIAL_H


#include <QObject>


class QextSerialPort;
class QString;


class IMcuCmd
    {
public:
    virtual ~IMcuCmd() {};
private:
    virtual QByteArray command() const = 0;
    virtual QString name() const = 0;
friend class McuSerial;
    };

class IMcuAns
    {
public:
    virtual ~IMcuAns() {};
private:
    virtual void receive(int &rnMin, int &rnMax,
        const QByteArray &aAnswerFragment) = 0;
    virtual QString name() const = 0;
friend class McuSerial;
    };

class IMcuCmdAns: public IMcuCmd, public IMcuAns
    {
public:
    virtual ~IMcuCmdAns() {};
    };


class McuSerial : public QObject
    {
    Q_OBJECT

public:
    McuSerial();
    virtual ~McuSerial();

    bool open(const QString &qsPort);
    void close();

    bool standardSendRecv(IMcuCmdAns *pCmdAns);

    bool send(IMcuCmd *pCmd);
    bool recv(IMcuAns *pAns);
    void flush();

signals:
    void log(const QString &qsText);

private:
    static QString hex(const QByteArray &aData);
    static void msleep(int nMsecs);

private:
    QextSerialPort *m_pSerPort;
    bool m_bOpen;
    QString m_qsLastCmd;
    };


#endif // ASPECT_MCUSERIAL_H

